# -*- coding:utf-8 -*-
#---------------------------------------------------------------------
#
# Visor Geogrfico
# Copyright (C) 2009 Germn Carrilo (GeoTux)
#
#---------------------------------------------------------------------
#
# licensed under the terms of GNU GPL 2
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this software. If not, see <http://www.gnu.org/licenses/>.
#
#---------------------------------------------------------------------

import re

from PyQt4.QtCore import Qt, QObject, SIGNAL, QSettings, QVariant
from PyQt4.QtGui import *

from utilidades import Utilidades

class DialogoCarguePostGIS( QDialog ):
    """ Provee la interfaz de la herramienta Cargar capa PostGIS """

    instancia = None # Variable para controlar la nica instancia

    class SingletonHelper:
        """ Provee un mtodo factory para el Singleton DialogoCarguePostGIS """
        def __call__( self, *args, **kw ) :

            if DialogoCarguePostGIS.instancia is None :
                object = DialogoCarguePostGIS( args[ 0 ] ) # Parent
                DialogoCarguePostGIS.instancia = object

            return DialogoCarguePostGIS.instancia

    obtenerInstancia = SingletonHelper() # Mtodo para obtener la instancia

    def __init__( self, parent=None ):

        if DialogoCarguePostGIS.instancia:
            raise RuntimeError, 'Solo se permite una instancia de esta clase!'

        QDialog.__init__( self, parent )

        self.utils = Utilidades.obtenerInstancia() # Obtener instancia de Utilidades

        # Diagramacin de la Interfaz
        self.setWindowTitle( 'Cargar capa de PostGIS' )

        lblServidor = QLabel( 'Servidor:' )
        lblPuerto = QLabel( 'Puerto:' )
        lblBd = QLabel( 'Base de datos:' )
        lblUsuario = QLabel( 'Usuario:' )
        lblContrasena = QLabel( 'Contrasea:' )
        lblTabla = QLabel( 'Tabla:' )

        self.__servidorEdit = QLineEdit( "localhost" )
        self.__puertoEdit = QLineEdit( "5432" )
        self.__bdEdit = QLineEdit( "trasmisig" )
        self.__usuarioEdit = QLineEdit( "usuario" )
        self.__contrasenaEdit = QLineEdit( self.utils.decStr( "dQBzAHUAYQByAGkAbwA=" ) )
        self.__contrasenaEdit.setEchoMode( QLineEdit.Password )

        self.cboTabla = QComboBox() # ComboBox para mostrar tablas disponibles
        self.cboTabla.setObjectName( "cboTabla" )

        self.btnConectar = QPushButton( QIcon( ":/iconosBotones/imagenes/conn.png" ),
            "Conectar" )

        buttonBox = QDialogButtonBox()
        self.btnAceptar = buttonBox.addButton( 'Aceptar',
            QDialogButtonBox.AcceptRole )
        self.btnAceptar.setEnabled( False )
        buttonBox.addButton( 'Cancelar', QDialogButtonBox.RejectRole )

        QObject.connect( self.btnConectar, SIGNAL( "clicked()" ), self.__conectar )
        QObject.connect( buttonBox, SIGNAL( "accepted()" ), self.__aceptar )
        QObject.connect( buttonBox, SIGNAL( "rejected()" ), self.cerrar )

        grid = QGridLayout()
        grid.setSpacing( 0 )

        grid.addWidget( lblServidor, 1, 0 )
        grid.addWidget( self.__servidorEdit, 1, 1 )

        grid.addWidget( lblPuerto, 2, 0 )
        grid.addWidget( self.__puertoEdit, 2, 1 )

        grid.addWidget( lblBd, 3, 0 )
        grid.addWidget( self.__bdEdit, 3, 1 )

        grid.addWidget( lblUsuario, 4, 0 )
        grid.addWidget( self.__usuarioEdit, 4, 1 )

        grid.addWidget( lblContrasena, 5, 0 )
        grid.addWidget( self.__contrasenaEdit, 5, 1 )

        grid.setRowStretch( 6,1 )

        grid.addWidget( self.btnConectar, 7, 0,1,2, Qt.AlignCenter )

        grid.setRowStretch( 8,1 )

        grid.addWidget( lblTabla, 9, 0 )
        grid.addWidget( self.cboTabla, 9, 1 )

        grid.setRowStretch( 10,1 )

        grid.addWidget( buttonBox,11,0,1,2, Qt.AlignCenter )

        self.setLayout( grid )
        self.setFixedSize( 260, 275 )

        self.resultado = False # Para conocer si el usuario acept o cancel

    def __conectar( self ):
        """ Valida los parmetros de conexin y carga las capas disponibles """
        validacion_puerto = re.compile( "^(\d{4})$" )
        validacion_cadenas = re.compile( "^(\S)*\S$" ) #No espacios, len(cadena)>0

        self.cboTabla.clear() # Inicializar cboTabla

        if validacion_cadenas.match( str( self.__servidorEdit.text() ) ):
            if validacion_puerto.match( str( self.__puertoEdit.text() ) ):
                if validacion_cadenas.match( str( self.__bdEdit.text() ) ):
                    if validacion_cadenas.match( str( self.__usuarioEdit.text() ) ):
                        if validacion_cadenas.match( str( self.__contrasenaEdit.text() ) ):

                            self.servidor = self.__servidorEdit.text()
                            self.puerto = self.__puertoEdit.text()
                            self.bd = self.__bdEdit.text()
                            self.usuario = self.__usuarioEdit.text()
                            self.contrasena = self.__contrasenaEdit.text()

                            # Cargar capas disponibles en la BD
                            from dialogos.conexion_postgis import ConexionPgSQL
                            conn =  ConexionPgSQL( self.servidor, self.puerto,
                                self.bd, self.usuario, self.contrasena )

                            if conn.conectar():
                                self.aDatosBD = conn.consultarCapas()

                                if self.aDatosBD:
                                    for i in range( 0, len( self.aDatosBD ) ):

                                        geom = self.aDatosBD[ i ][ 3 ]
                                        print "***",geom,"***"
                                        if geom == "POINT" or geom == 'MULTIPOINT':
                                            icono= QIcon( ":/otrosIconos/imagenes/mIconPointLayer.png" )
                                        elif geom == "LINESTRING" or geom == "MULTILINESTRING" or geom == "LINESTRINGM":
                                            icono= QIcon( ":/otrosIconos/imagenes/mIconLineLayer.png" )
                                        elif geom == "POLYGON" or geom == "MULTIPOLYGON":
                                            icono= QIcon( ":/otrosIconos/imagenes/mIconPolygonLayer.png" )
                                        else:
                                            raise RuntimeError, "Geometra desconocida: " + str( geom )

                                        self.cboTabla.addItem( icono, self.aDatosBD[ i ][ 1 ] )

                                    self.guardarConexion()
                                    self.btnAceptar.setEnabled( True ) # Habilitar
                                else:
                                    self.btnAceptar.setEnabled( False ) # Deshabilitar
                                    self.utils.mostrarMensaje( self, 'Advertencia',
                                        'No existen tablas espaciales en la ' \
                                        + 'base de datos que\npuedas visualizar' \
                                        + ' con los parmetros ingresados.',
                                        QMessageBox.Warning )
                            else:
                                self.btnAceptar.setEnabled( False ) # Deshabilitar
                                self.utils.mostrarMensaje( self, 'Advertencia',
                                    'No se ha podido establecer la conexin ' \
                                    + 'con\nla base de datos que especificaste.' \
                                    + '\n\nRevisa los parmetros de conexin.',
                                    QMessageBox.Warning )
                        else:
                            self.btnAceptar.setEnabled( False ) # Deshabilitar
                            self.utils.mostrarMensaje( self, 'Advertencia',
                                'Has llenado mal el campo <i>Contrasea</i>, revsalo.',
                                QMessageBox.Warning )
                    else:
                        self.btnAceptar.setEnabled( False ) # Deshabilitar
                        self.utils.mostrarMensaje( self, 'Advertencia',
                            'Has llenado mal el campo <i>Usuario</i>, revsalo.',
                            QMessageBox.Warning )
                else:
                    self.btnAceptar.setEnabled( False ) # Deshabilitar
                    self.utils.mostrarMensaje( self, 'Advertencia',
                        'Has llenado mal el campo <i>Base de datos</i>, revsalo.',
                        QMessageBox.Warning )
            else:
                self.btnAceptar.setEnabled( False ) # Deshabilitar
                self.utils.mostrarMensaje( self, 'Advertencia',
                    'Has llenado mal el campo <i>Puerto</i>, revsalo.',
                    QMessageBox.Warning )
        else:
            self.btnAceptar.setEnabled( False ) # Deshabilitar
            self.utils.mostrarMensaje( self, 'Advertencia',
                'Has llenado mal el campo <i>Servidor</i>, revsalo.',
                QMessageBox.Warning )

    def guardarConexion( self ):
        """ Guarda los parmetros de conexin en QSettings """
        settings = QSettings()
        baseKey = "/PostgreSQL/connections/"
        settings.setValue( baseKey + "selected", QVariant( self.bd ) )
        baseKey += self.bd
        settings.setValue( baseKey + "/host", QVariant( self.servidor ) )
        settings.setValue( baseKey + "/database", QVariant( self.bd ) )
        settings.setValue( baseKey + "/port", QVariant( self.puerto ) )
        settings.setValue( baseKey + "/username", QVariant( self.usuario ) )
        settings.setValue( baseKey + "/password", QVariant( self.contrasena ) )
        settings.setValue( baseKey + "/publicOnly",  QVariant( False ) ) # Public schema only
        settings.setValue( baseKey + "/geometryColumnsOnly", QVariant( False ) ) # Geom columns only
        settings.setValue( baseKey + "/save", QVariant( "True" ) ) # Store password
        settings.setValue( baseKey + "/sslmode", QVariant( "allow" ) )

    def __aceptar( self ):
        self.resultado = True # Aceptado

        # Estas asignaciones permite validar cualquier cambio posterior a la conexion
        self.servidor = self.__servidorEdit.text()
        self.puerto = self.__puertoEdit.text()
        self.bd = self.__bdEdit.text()
        self.usuario = self.__usuarioEdit.text()
        self.contrasena = self.__contrasenaEdit.text()

        # Datos de la tabla a cargar
        self.esquema = self.aDatosBD[ self.cboTabla.currentIndex() ][ 0 ]
        self.tabla = self.aDatosBD[ self.cboTabla.currentIndex() ][ 1 ]
        self.geom = self.aDatosBD[ self.cboTabla.currentIndex() ][ 2 ]
        self.close()

    def mostrar(self):
        self.resultado = False # Inicializar
        self.exec_()

    def cerrar(self):
        self.resultado = False # Cancelado
        self.close()
